// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once
#include <string>
#include <vector>
#include <sstream>
#include <unordered_map>
#include "compile_defines.h"
#include "es_kernel_public_defines.h"
#include "es_nn_common.h"

#define TENSOR_SHAPE_SET_WITH_IDX(dst_shape, src_shape, idx) \
    dst_shape.N = src_shape[idx].N;                          \
    dst_shape.C = src_shape[idx].C;                          \
    dst_shape.H = src_shape[idx].H;                          \
    dst_shape.W = src_shape[idx].W;

// Define constants for alignment
const int32_t ALIGNMENT = 128;
const int32_t NO_ALIGNMENT = 0;

// Function templates to convert string to a specific type
template <typename T>
T convert_to_type(const std::string& value, T);

// Macro to parse and assign parameters from YAML map
#define PARSE_PARAM(key, Type)                               \
    {                                                        \
        auto value = convert_to_type(yml_map[#key], Type()); \
        problem->OPArgs[#key] = malloc(sizeof(Type));        \
        *((Type*)(problem->OPArgs[#key])) = value;           \
    }

// retrieve an operation argument of a specific type
template <typename T>
T get_op_arg(const ES_DSP_OP_PROBLEM_S& problem, const std::string& key)
{
    auto iter = problem.OPArgs.find(key);
    if (iter != problem.OPArgs.end()) {
        return *((T*)(iter->second));
    } else {
        throw std::runtime_error("Key not found: " + key);
    }
}

// Function to calculate the size of a tensor based on its shape and element size
inline int32_t calc_tensor_size(const ES_TENSOR_SHAPE_S& shape, int32_t elem_size, int32_t align)
{
    return shape.N * shape.C * shape.H * shape.W * shape.C0 * elem_size + align;
}

extern void set_kernel_name(const ES_DSP_OP_PROBLEM_S& problem, ES_DSP_OPERATOR_DESC_S* desc,
                            const std::string& tail_str);

// generate the DSP operator descriptor based on the problem specification
template <typename ConfigType>
int32_t gen_dsp_op_desc(ES_DSP_OP_PROBLEM_S& problem, ES_DSP_OPERATOR_DESC_S** desc,
                        std::vector<int32_t>& ext_params_size)
{
    if (desc == nullptr) {
        ES_HOST_ERROR("Output parameter desc is a nullptr.\n");
        return -1;
    }

    // Step 1: get total size: include config size、 external params size、 input size、output size
    int32_t total_size = sizeof(ES_DSP_OPERATOR_DESC_S);

    // Step 1.1: add config size
    total_size += sizeof(ConfigType);

    // Step 1.2: add external params size
    for (int32_t param_size : ext_params_size) {
        total_size += param_size;
    }

    // Step 2: malloc desc buffer
    ES_DSP_OPERATOR_DESC_S* op_desc = (ES_DSP_OPERATOR_DESC_S*)malloc(total_size);
    if (op_desc == nullptr) {
        ES_HOST_ERROR("Memory allocation failed for ES_DSP_OPERATOR_DESC_S.\n");
        return -1;
    }
    memset(op_desc, 0, total_size);
    op_desc->totalSize = total_size;

    // Step 3: determine the buffer size of each section
    const int32_t config_num = 1;
    int32_t ext_params_num = ext_params_size.size();
    op_desc->bufferCntCfg = config_num + ext_params_num;

    op_desc->bufferCntInput = problem.inputArgs.size();
    op_desc->bufferCntOutput = problem.outputArgs.size();
    *desc = op_desc;

    // Step 3.1: determine config size
    op_desc->bufferSize[0] = sizeof(ConfigType);

    int32_t index = 1;
    // Step 3.2: determine external params size: include lut tabla and others
    for (int32_t param_size : ext_params_size) {
        if (index >= BUFFER_CNT_MAXSIZE) {
            ES_HOST_ERROR("External buffer count exceeds the maximum limit.\n");
            free(op_desc);
            return -1;
        }
        op_desc->bufferSize[index++] = param_size;
    }

    // Step 3.3: determine input data size
    for (const auto& arg : problem.inputArgs) {
        if (index >= BUFFER_CNT_MAXSIZE) {
            ES_HOST_ERROR("Input buffer count exceeds the maximum limit.\n");
            free(op_desc);
            return -1;
        }
        op_desc->bufferSize[index++] = calc_tensor_size(arg.shape, DataTypeInfoMap[arg.precision].size, NO_ALIGNMENT);
    }

    // Step 3.4: determine output data size
    for (const auto& arg : problem.outputArgs) {
        if (index >= BUFFER_CNT_MAXSIZE) {
            ES_HOST_ERROR("Output buffer count exceeds the maximum limit.\n");
            free(op_desc);
            return -1;
        }
        op_desc->bufferSize[index++] = calc_tensor_size(arg.shape, DataTypeInfoMap[arg.precision].size, NO_ALIGNMENT);
    }

    return 0;
}

template <typename T>
static void get_array_from_yml(T* arr_table_ptr, const std::string& yml_table)
{
    std::string new_string;
    for (auto& c : yml_table) {
        if (c != '[' && c != ']') {
            new_string += c;
        }
    }

    int len_table = 0;
    std::stringstream ss(new_string);
    std::string token;
    while (std::getline(ss, token, ',')) {
        std::stringstream val;
        val << token;
        T v;
        val >> v;
        arr_table_ptr[len_table++] = v;
    }
}

template <class T>
std::vector<T> read_vec_from_yml(const std::string& str)
{
    std::string new_string;
    for (auto& c : str) {
        if (c != '[' && c != ']') {
            new_string += c;
        }
    }

    std::vector<T> vec;
    std::stringstream ss(new_string);
    std::string token;
    while (std::getline(ss, token, ',')) {
        std::stringstream val;
        val << token;
        T v;
        val >> v;
        vec.push_back(v);
    }
    return vec;
}

static std::unordered_map<ES_INTER_FLAG_E, std::string> resize_method_to_string_map = {
    {ES_INTER_LINEAR, "bilinear"}, {ES_INTER_NEAREST, "nearest"}, {ES_INTER_AREA, "area"}, {ES_INTER_CUBIC, "bicubic"}};

static std::unordered_map<std::string, ES_INTER_FLAG_E> resize_string_to_method_map = {
    {"bilinear", ES_INTER_LINEAR}, {"nearest", ES_INTER_NEAREST}, {"area", ES_INTER_AREA}, {"bicubic", ES_INTER_CUBIC}};

extern int32_t common_parse_kvs(ES_DSP_OP_PROBLEM_S* desc, std::unordered_map<std::string, std::string> yml_map);
extern int32_t common_desc2_parsed_desc(ES_DSP_OP_PROBLEM_S problem, dsp_problem_kernel_parsed_desc* parsed_desc);
extern int32_t allocate_desc(ES_DSP_OPERATOR_DESC_S** desc, dsp_problem_kernel_parsed_desc parsed_desc);
extern int32_t write_desc(ES_DSP_OPERATOR_DESC_S* desc, dsp_problem_kernel_parsed_desc parsed_desc);
extern int32_t write_desc_bin(ES_DSP_OPERATOR_DESC_S* desc, ES_DSP_OP_PROBLEM_S problem);
extern void dump_ES_DSP_OP_PROBLEM_S(dsp_problem_kernel_parsed_desc desc);
extern void set_tensor_paras(es_dsp_tensor* tensor, ES_TENSOR_SHAPE_S* parsed_tensor_desc, int index);
extern void add_tensors_to_config(ES_DSP_OP_PROBLEM_S problem, es_dsp_tensor* in_tensor_vec,
                                  es_dsp_tensor* out_tensor_vec);
bool check_types(ES_DATA_PRECISION_E in, ES_DATA_PRECISION_E out,
                 std::vector<std::pair<ES_DATA_PRECISION_E, ES_DATA_PRECISION_E>> precisionMap);