// Copyright © 2024 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#pragma once

#include "es_nn_common.h"
#include "internal/dsp_kernel_common.h"

// follow params using in subgraph or complete network
#define OP_NUM_MAX 64            /* the max op num of net */
#define OUT_CHANNEL_NUM_MAX 2048 /* equal to the num of conv kernel weight */
#define FUSION_OP_NUM_MAX 8      /* the max op num of one fusion op */
#define LUT_LEN_MAX 256          /* the max length of the activation function lut */
#define IN_TENSOR_NUM_MAX 4      /* max num of input tesnors per op */
#define OUT_TENSOR_NUM_MAX 2

typedef enum {
    N = 0,   // batch size
    C = 1,   // number of C0
    H = 2,   // height
    W = 3,   // width
    C0 = 4,  // the atomic unit of channel
    Cs = 5,  // the real channels Cs <= C*C0 and C = ceil(Cs/C0)
} ES_TENSOR_SHAPE_E;

/* compiler use this structure describing pipeline stages , number of available devices and so on. */
typedef struct {
    ES_S32 deviceNum;                  // number of available device
    ES_S32 pipelineTileNum;            // pipeline stage
    ES_TENSOR_SHAPE_E axis;            // the axis of tile
    ES_DSP_TENSOR_DESC_S dspTileInfo;  // pipeline tile info
} ES_PIPELINE_OPTION_S;

/* describe the cost info */
typedef struct {
    ES_S32 deviceUsedNum;  // The actual number of devices in use
    ES_FLOAT latency;
    ES_FLOAT energy;  // cost info
} ES_DSP_COST_INFO_S;

/* compiler emit operator in pipeline*/
typedef struct {
    ES_S32 diviceId;
    ES_S32 pipelineStage;
    ES_S32 inputOffset;
    ES_S32 outputOffset;
    ES_DSP_TENSOR_DESC_S inputDspTileInfo;
    ES_DSP_TENSOR_DESC_S outputDspTileInfo;
} ES_DSP_EMIT_INFO_S;

/* compiler emit operators in one pipeline struct*/
typedef struct {
    ES_S32 diviceId;
    ES_S32 pipelineStage;
    ES_S32 inputOffset[MAX_PIPELINE_NUM];
    ES_S32 outputOffset[MAX_PIPELINE_NUM];
    ES_DSP_TENSOR_DESC_S inputDspTileInfo[MAX_PIPELINE_NUM];
    ES_DSP_TENSOR_DESC_S outputDspTileInfo[MAX_PIPELINE_NUM];
} ES_DSP_EMIT_MULTIPLE_INFO_S;

/* Instruct dsp to go through different impl */
typedef enum {
    /* basic op */
    ES_CONV_NCHW_LUT = 0,
    ES_CONV_NCHW_RELU = 1,
    ES_CONV_NHWC_LUT = 2,
    ES_CONV_NHWC_RELU = 3,
    ES_DEPTHWISE_CONV_NCHW_LUT = 4,
    ES_DEPTHWISE_CONV_NCHW_RELU = 5,
    ES_DEPTHWISE_CONV_NHWC_LUT = 6,
    ES_DEPTHWISE_CONV_NHWC_RELU = 7,
    ES_POINTWISE_CONV_NCHW_NHWC_RELU = 8,
    ES_POINTWISE_CONV_NCHW_NCHW_RELU = 9,
    ES_POINTWISE_CONV_NHWC_LUT = 10,
    ES_POINTWISE_CONV_NHWC_RELU = 11,
    ES_ADD_ = 12,
} ES_DSP_BASIC_OP_TYPE_E;

/* the input or output layout of per op */
typedef enum {
    ES_DSP_NCHW = 0,
    ES_DSP_NHWC = 1,
    ES_DSP_NCHWC0 = 2,  // C0 = 32 bytes
} ES_DSP_OP_LAYOUT_E;

/* Common config using in dsp tiling */
typedef struct {
    ES_S32 input_dram_size;   // the buf size to store input in dram
    ES_S32 output_dram_size;  // the buf size to store output in dram
    ES_U32 dma_params_ptr;    // to store the addr of dma params for each tile
    ES_S32 op_tiling_cnt;     // the num of tile in this op
} ES_DSP_COMMON_TILING_S;

/* Config of add op */
typedef struct {
    ES_S32 upper_bound;  // unify relu and relu6   注：add 也带激活函数
    ES_S32 lower_bound;

    /* add op params ----> scale info */
    ES_S32 input0_scale;
    ES_S32 input1_scale;
    ES_S32 right_shift;

    /* op public info */
    ES_DSP_OP_LAYOUT_E op_layout;
    ES_DSP_TENSOR_DESC_S input_tensor[2];  //  注：这里内部的 shape 指的是编译器切分后的 shape
    ES_DSP_TENSOR_DESC_S output_tensor[1];
} __attribute((aligned(4))) ES_DSP_ADD_OP_S;

/* Config of conv op */
typedef struct {
    /* op public info */
    ES_DSP_OP_LAYOUT_E op_layout;
    ES_S32 right_shift;
    ES_S32 pack_shift;
    ES_S32 out_shift;
    ES_S32 upper_bound;  // unify relu and relu6
    ES_S32 lower_bound;

    /* inner info  ----> fill in by DSP, instead of compiler */
    ES_U32 weight_ddr_ptr;  // using in conv op
    ES_U32 bias_ddr_ptr;    // using in conv op

    /* conv op params ----> weight and bias info */
    ES_S32 kernel_shape[2];
    ES_S32 pad_left;
    ES_S32 pad_right;
    ES_S32 pad_up;
    ES_S32 pad_down;
    ES_S32 strides[2];

    /* conv op params ----> scale info */
    ES_U16 weight_scale[OUT_CHANNEL_NUM_MAX];
    ES_S8 act_lut[LUT_LEN_MAX];  //  using in silu and so on

    ES_DSP_TENSOR_DESC_S input_tensor;
    ES_DSP_TENSOR_DESC_S output_tensor;
} __attribute((aligned(4))) ES_DSP_CONV_OP_S;

/* dictionary of per op */
typedef struct {
    ES_DSP_BASIC_OP_TYPE_E op_type;
    ES_S32 in_data_id[IN_TENSOR_NUM_MAX];
    ES_S32 out_data_id[OUT_TENSOR_NUM_MAX];
    ES_S32 cfg_size;  // equal sizeof(each op_type config)
    ES_S32 cfg_offset;
    ES_S32 weight_offset;
    ES_S32 bias_offset;
} __attribute((aligned(4))) ES_DSP_OP_DICT_S;

/* The header information of net */
typedef struct {
    ES_S32 total_buf_size;
    ES_S32 total_op_num;
    ES_DSP_OP_DICT_S op_dict_arr[OP_NUM_MAX];
    ES_CHAR params_data[0];  // store op config, weight and bias
} __attribute((aligned(4))) ES_DSP_NET_DESC_S;
