// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESWIN_DSP_API_H__
#define __ESWIN_DSP_API_H__

#include "es_type.h"
#include "es_dsp_types.h"
#include "es_dsp_op_types.h"
#include "es_dsp_error.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif

/**
 * @brief Get DSP version information.
 *
 * @param[out] version Version number of the DSP.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_GetVersion(ES_U64* version);

/**
 * @brief Get DSP version string.
 *
 * @param[out] version Buffer to store the version information
 * @param[in] maxSize Maximum size of the buffer
 * @return Returns the status code of the operation; 0 if successful, an error code otherwise
 */
ES_S32 ES_DSP_GetVersionString(ES_CHAR* version, ES_U32 maxSize);

/**
 * @brief Get DSP capability
 *
 * @param[out] capability Structure to store DSP capability information
 * @return Returns the status code of the operation; 0 if successful, an error code otherwise
 */
ES_S32 ES_DSP_GetCapability(ES_DSP_Capability_S* capability);

/**
 * @brief Set the log level for DSP runtime logging.
 *
 * @param[in] level Log level for DSP logging.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_SetLogLevel(ES_U32 level);

/**
 * @brief Open the specified DSP device.
 *
 * @param[in] dspId DSP device ID.
 * @param[in] opLoadPolicy Load policy for specified operators on this DSP.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_Open(ES_DSP_ID_E dspId, ES_DSP_LOAD_POLICY_E opLoadPolicy);

/**
 * @brief Close the specified DSP device.
 *
 * @param[in] dspId DSP device ID.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_Close(ES_DSP_ID_E dspId);

/**
 * @brief Load an operator with a specified name and library path onto the specified DSP device and return the operator
 * handle.
 *
 * @param[in] dspId DSP device ID.
 * @param[in] operatorName Operator name.
 * @param[in] operatorLibDir Operator library path.
 * @param[out] operatorHandle Operator handle.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LoadOperator(ES_DSP_ID_E dspId, const ES_CHAR* operatorName, const ES_CHAR* operatorLibDir,
                           ES_DSP_HANDLE* operatorHandle);

/**
 * @brief Unload the operator on the specified DSP device using the loaded operator handle.
 *
 * @param[in] dspId DSP device ID.
 * @param[in] operatorHandle Operator handle.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_UnloadOperator(ES_DSP_ID_E dspId, const ES_DSP_HANDLE operatorHandle);

/**
 * @brief Synchronously submit operator tasks to the specified DSP device.
 *
 * @param[in] dspId DSP device ID.
 * @param[in] operatorTask Operator task.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_SubmitTask(ES_DSP_ID_E dspId, ES_DSP_TASK_S* operatorTask);

/**
 * @brief Asynchronously submit operator tasks to the specified DSP device and return the task handle.
 *
 * @param[in] dspId DSP device ID.
 * @param[in] operatorTask Operator task.
 * @param[out] taskHandle Task handle.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_SubmitTaskAsync(ES_DSP_ID_E dspId, ES_DSP_TASK_S* operatorTask, ES_DSP_HANDLE* taskHandle);

/**
 * @brief Check whether the operator task on the specified DSP device is completed based on the task handle.
 *
 * @param[in] dspId DSP device ID.
 * @param[in] taskHandle Task handle.
 * @param[in] block Whether to block the query.
 * @param[out] taskFinish Whether the task has completed.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_QueryTask(ES_DSP_ID_E dspId, ES_DSP_HANDLE taskHandle, ES_BOOL block, ES_BOOL* taskFinish);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif

#endif