// Copyright © 2023 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESWIN_DSP_LOW_LEVEL_API_H__
#define __ESWIN_DSP_LOW_LEVEL_API_H__

#include "es_type.h"
#include "es_dsp_types.h"
#include "es_dsp_op_types.h"
#include "es_dsp_error.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif

/**
 * @brief Get DSP version information.
 *
 * @param[out] version Version number of the DSP.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_GetVersion(ES_U64 *version);

/**
 * @brief Get DSP version string.
 *
 * @param[out] version Buffer to store the version information
 * @param[in] maxSize Maximum size of the buffer
 * @return Returns the status code of the operation; 0 if successful, an error code otherwise
 */
ES_S32 ES_DSP_LL_GetVersionString(ES_CHAR *version, ES_U32 maxSize);

/**
 * @brief Get DSP capability
 *
 * @param[out] capability Structure to store DSP capability information
 * @return Returns the status code of the operation; 0 if successful, an error code otherwise
 */
ES_S32 ES_DSP_LL_GetCapability(ES_DSP_Capability_S *capability);

/**
 * @brief Set the log level for DSP runtime logging.
 *
 * @param[in] level Log level for DSP logging.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_SetLogLevel(ES_U32 level);

/**
 * @brief Open the specified DSP device.
 *
 * @param[in] dspId DSP device ID.
 * @param[out] dspFd DSP device descriptor.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_Open(ES_DSP_ID_E dspId, ES_S32 *dspFd);

/**
 * @brief Close the specified DSP device.
 *
 * @param[in] dspFd DSP device descriptor.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_Close(ES_S32 dspFd);

/**
 * @brief Load an operator onto the specified DSP device with the given name and library path,
 *        and return the operator handle.
 *
 * @param[in] dspFd DSP device descriptor.
 * @param[in] operatorName Operator name.
 * @param[in] operatorLibDir Operator library path.
 * @param[out] operatorHandle Operator handle.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_LoadOperator(ES_S32 dspFd, const ES_CHAR *operatorName, const ES_CHAR *operatorLibDir,
                              ES_DSP_HANDLE *operatorHandle);

/**
 * @brief Unload the operator on the specified DSP device using the provided operator handle.
 *
 * @param[in] dspFd DSP device descriptor.
 * @param[in] operatorHandle Operator handle.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_UnloadOperator(ES_S32 dspFd, const ES_DSP_HANDLE operatorHandle);

/**
 * @brief Prepare the DMA buffer for a specified DSP device.
 *
 * @param[in] dspFd DSP device descriptor.
 * @param[in] buffer DMA buffer data structure descriptor consisting of fd, offset, and size.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_PrepareDMABuffer(ES_S32 dspFd, ES_DEV_BUF_S buffer);

/**
 * @brief Release the DMA buffer for a specified DSP device.
 *
 * @param[in] dspFd DSP device descriptor.
 * @param[in] fd DMA buffer file descriptor.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_UnprepareDMABuffer(ES_S32 dspFd, ES_U64 fd);

/**
 * @brief Synchronously submit operator tasks to the specified DSP device.
 *
 * @param[in] dspFd DSP device descriptor.
 * @param[in] operatorTask Operator task.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_SubmitTask(ES_S32 dspFd, ES_DSP_TASK_S *operatorTask);

/**
 * @brief Asynchronously submit operator tasks to the specified DSP device and return the task handle.
 *
 * @param[in] dspFd DSP device descriptor.
 * @param[in] operatorTask Operator task.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_SubmitTaskAsync(ES_S32 dspFd, ES_DSP_TASK_S *operatorTask);

/**
 * @brief Execute the callback function registered during operator task submission.
 *
 * @param[in] dspFd DSP device descriptor.
 * @param[in] timeout (ms) Maximum time to wait for task completion when no task is completed,
 *                      -1 to block until a task is completed.
 * @return Returns the execution status code: ES_DSP_SUCCESS for success, others for failure.
 */
ES_S32 ES_DSP_LL_ProcessReport(ES_S32 dspFd, ES_S32 timeout);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif

#endif